/* File structures for Aurora Aurora Videographics System
 * File format analyzed and reversed by Mihai Gaitos in 2015
 * first release, 2015-09-27
 */

#define AURSIG 0xb2cb /* this looks like file signature, first 2 bytes, little endian */

/* im header array starts 8 bytes into the file */
typedef struct
{
	uint16_t	t1;   /* section type 0x28 = animation? ; 0x20 = palette, 0x10 = rta header -> rta -> rtb image data */
	uint16_t	t2;   /* this is 0x01 for palette and regular data but 0x80 for epilogue; fonts have various values here, including what appears to be ASCII codes */
	uint16_t	offh; /* offset high word */
	uint16_t	offl; /* offset low word */
} imheada;


/* runtab data - .im image data, RLE */
typedef struct
{
	uint8_t val;
	uint8_t run;
} rtb;


/* runtab array - points to RLE data */
typedef struct {
	uint16_t	t1;	/* 0x0007 = index into RLE, no other record type found */
	uint16_t	offh;	/* offset high word */
	uint16_t	offl;	/* offset low word */
	} rta;

/* header of runtab array - contains pointer to array */
typedef struct
{
	uint16_t a1;
	uint16_t a2;
	int16_t a3;
	int16_t a4;	/* unknown (signed?) value */
	uint16_t s1;	/* width */
	uint16_t s2;	/* height */
	uint8_t unkn[32];/* this is identical for all available picture files
			   but there are some differences in various sections of fonts */
	uint16_t rtabegh; /* HIGH word of start of rta */
	uint16_t rtabegl; /* LOW word of start of rta */
} rtah;


/* palette header */
typedef struct
{
	uint16_t res0; /* always 0 */
	uint16_t ccnt; /* I assume no. of color entries; it's always 0x80 */
} palh;

