/* mplot.c - memory specific graphic functions used for plotting.
 * generic functions are in gfunctions.c
 * "Alpha" state. Use at your own risk!
 * Copyright 2012 Mihai Gaitos, mihaig@hawk.ro
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

typedef struct M_Surface {
        uint16_t w;
		uint16_t h;
		uint8_t bpp;	/* bpp is bits per pixel */
		uint8_t BytesPerPixel;
        void *pixels;
} M_Surface;

/* only 32bpp surfaces are supported for now */
M_Surface *M_CreateSurface(uint32_t x,uint32_t y,uint8_t bpp)
{
if(bpp!=32)
	{
	fprintf(stderr,"Only 32bpp depth allowed! Surface created with 32bpp (requested %dbpp)!\n",bpp);
	bpp=32;
	}
uint8_t Bpp=(bpp+7)>>3;
unsigned long size=sizeof (M_Surface)+x*y*Bpp;
M_Surface *s=malloc(size);
s->w=x;
s->h=y;
s->bpp=bpp;
s->BytesPerPixel=Bpp;
s->pixels=(uint8_t *)s+sizeof(M_Surface);
return s;
}


int getsurfacesize(M_Surface *screen,unsigned int *x, unsigned int *y)
{
*x=screen->w;
*y=screen->h;
}


int M_GetRGB(uint32_t pixel,uint8_t *r,uint8_t *g,uint8_t *b)
{
*r=(pixel>>16)&0xff;
*g=(pixel>>8)&0xff;
*b=pixel&0xff;
}

uint32_t M_SetRGB(uint8_t r, uint8_t g, uint8_t b)
{
uint32_t res=r<<16;
res|=g<<8;
res|=b;
return res;
}


int getpixel(M_Surface *screen,int x,int y,pixel_t *np)
{
if(x<0 || x>=screen->w)
	{
	if(y<0 || y>=screen->h)
		return Y_OUT_OF_RANGE | X_OUT_OF_RANGE;
	else
		return X_OUT_OF_RANGE;
	}
if(y<0 || y>=screen->h)
	return Y_OUT_OF_RANGE;
uint32_t pixel;
uint8_t *bits;
bits = (uint8_t *)screen->pixels+y*screen->w*screen->BytesPerPixel+x*screen->BytesPerPixel;

pixel=*(uint32_t *)bits;
M_GetRGB(pixel,&np->r,&np->g,&np->b);
return G_OK;
}

int plot(M_Surface *screen,int x,int y,pixel_t *np,int mod)
{
if(x<0 || x>=screen->w)
	{
	if(y<0 || y>=screen->h)
		return Y_OUT_OF_RANGE | X_OUT_OF_RANGE;
	else
		return X_OUT_OF_RANGE;
	}
if(y<0 || y>=screen->h)
	return Y_OUT_OF_RANGE;

uint32_t   pixel;
uint8_t   *bits;
if((mod&PLOTMASK)!=0)
	{
	pixel_t op;
	getpixel(screen,x,y,&op);
	switch(mod&PLOTMASK)
		{
		case PX_ADD:
			if(op.r+np->r>255)
				op.r=255;
			else
				op.r+=np->r;
			if(op.g+np->g>255)
				op.g=255;
			else
				op.g+=np->g;
			if(op.b+np->b>255)
				op.b=255;
			else
				op.b+=np->b;
			break;
		}
	pixel = M_SetRGB(op.r, op.g, op.b);
	}
else
	pixel = M_SetRGB(np->r, np->g, np->b);
bits = (uint8_t *)screen->pixels+y*screen->w*screen->BytesPerPixel+x*screen->BytesPerPixel;
*((uint32_t *)(bits)) = (uint32_t)pixel;
return G_OK;
}

