/* sdlplot.c - SDL specific graphic functions used for plotting.
 * generic functions are in gfunctions.c
 * "Alpha" state. Use at your own risk!
 * Copyright 2012 Mihai Gaitos
 * Loosely based on SDL Programming Examples
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

int getsurfacesize(SDL_Surface *screen,unsigned int *x, unsigned int *y)
{
*x=screen->w;
*y=screen->h;
}

int getpixel(SDL_Surface *screen,int x,int y,pixel_t *np)
{
if(x<0 || x>=screen->w || y<0 || y>=screen->h)
	return 0;

if(SDL_MUSTLOCK(screen)!=0)
	if(SDL_LockSurface(screen)<0)
		return -1;

Uint32	pixel;
Uint8	*bits,bpp;

bpp = screen->format->BytesPerPixel;
bits = (Uint8 *)screen->pixels+y*screen->pitch+x*bpp;

switch(bpp)
	{
	case 1:
		pixel=*bits;
		break;
	case 2:
		pixel=*(Uint16 *)bits;
		break;
	case 3:
		{
		Uint8 r,g,b;
		r=*((bits)+screen->format->Rshift/8); 
		g=*((bits)+screen->format->Gshift/8);
		b=*((bits)+screen->format->Bshift/8);
		pixel=r<<screen->format->Rshift;
		pixel|=g<<screen->format->Gshift;
		pixel|=b<<screen->format->Bshift;
		fprintf(stderr,"%x\t%x\t%x\t=%x\n",r,g,b,pixel);
		}
		break;
	case 4:
		pixel=*(Uint32 *)bits;
		break;
	}
if(SDL_MUSTLOCK(screen)!=0)
	SDL_UnlockSurface(screen);
SDL_GetRGB(pixel,screen->format,&np->r,&np->g,&np->b);
return 1;
}

int plot(SDL_Surface *screen,int x,int y,pixel_t *np,int mod)
{
if(x<0 || x>=screen->w || y<0 || y>=screen->h)
	return 0;
Uint32   pixel;
Uint8   *bits, bpp;

if((mod&PLOTMASK)!=0)
	{
	pixel_t op;
	getpixel(screen,x,y,&op);
	switch(mod&PLOTMASK)
		{
		case PX_ADD:
			if(op.r+np->r>255)
				op.r=255;
			else
				op.r+=np->r;
			if(op.g+np->g>255)
				op.g=255;
			else
				op.g+=np->g;
			if(op.b+np->b>255)
				op.b=255;
			else
				op.b+=np->b;
			break;
		}
	pixel = SDL_MapRGB(screen->format, op.r, op.g, op.b);
	}
else
	pixel = SDL_MapRGB(screen->format, np->r, np->g, np->b);

bpp = screen->format->BytesPerPixel;

if(SDL_MUSTLOCK(screen)!=0)
	if(SDL_LockSurface(screen)<0)
		return -1;

bits = (Uint8 *)screen->pixels+y*screen->pitch+x*bpp;

/* Set the pixel */
switch(bpp)
	{
	case 1:
		*bits = (Uint8)pixel;
		break;
	case 2:
		*(Uint16 *)bits = (Uint16)pixel;
		break;
	case 3:
		{ /* Format/endian independent */
		Uint8 r, g, b;
		r = (pixel>>screen->format->Rshift)&0xFF;
		g = (pixel>>screen->format->Gshift)&0xFF;
		b = (pixel>>screen->format->Bshift)&0xFF;
		*((bits)+screen->format->Rshift/8) = r; 
		*((bits)+screen->format->Gshift/8) = g;
		*((bits)+screen->format->Bshift/8) = b;
		}
		break;
	case 4:
		*((Uint32 *)(bits)) = (Uint32)pixel;
		break;
	}
if(SDL_MUSTLOCK(screen)!=0)
	SDL_UnlockSurface(screen);
return 1;
}

